# -*- encoding: utf-8 -*-
import functools
from abjad.tools.sievetools.BaseResidueClass import BaseResidueClass


@functools.total_ordering
class ResidueClass(BaseResidueClass):
    r'''Residue class (or congruence class).

    Residue classes form the basis of Xenakis sieves. They can be used to
    make any complex periodic integer or boolean sequence as a
    combination of simple periodic sequences.

    ..  container:: example

        From the opening of Xenakis's *Psappha* for solo percussion:

        ::

            >>> RC = sievetools.ResidueClass

        ::

            >>> s1 = (RC(8, 0) | RC(8, 1) | RC(8, 7)) & (RC(5, 1) | RC(5, 3))
            >>> s2 = (RC(8, 0) | RC(8, 1) | RC(8, 2)) & RC(5, 0)
            >>> s3 = RC(8, 3)
            >>> s4 = RC(8, 4)
            >>> s5 = (RC(8, 5) | RC(8, 6)) & (RC(5, 2) | RC(5, 3) | RC(5, 4))
            >>> s6 = (RC(8, 1) & RC(5, 2))
            >>> s7 = (RC(8, 6) & RC(5, 1))

        ::

            >>> y = s1 | s2 | s3 | s4 | s5 | s6 | s7

        ::

            >>> y.get_congruent_bases(40)
                [0, 1, 3, 4, 6, 8, 10, 11, 12, 13, 14, 16, 17, 19, 20, 22,
                23, 25, 27, 28, 29, 31, 33, 35, 36, 37, 38, 40]

        ::

            >>> y.get_boolean_train(40)
                [1, 1, 0, 1, 1, 0, 1, 0, 1, 0, 1, 1, 1, 1, 1, 0, 1, 1, 0, 1,
                1, 0, 1, 1, 0, 1, 0, 1, 1, 1, 0, 1, 0, 1, 0, 1, 1, 1, 1, 0]

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_modulo',
        '_residue',
        )

    ### INITIALIZER ##

    def __init__(self, *args):
        if len(args) == 1:
            self._initialize_by_rc_instance(*args)
        elif len(args) == 2:
            self._initialize_by_modulo_and_residue(*args)
        elif len(args) == 0:
            self._initialize_by_modulo_and_residue(1, 0)
        else:
            message = 'can not intialize residue class: {!r}.'
            message = message.format(args)
            raise ValueError(message)

    ### SPECIAL METHODS ###

    def __eq__(self, expr):
        r'''Is true when `expr` is a residue class with module and residue equal
        to those of this residue class. Otherwise false.

        Returns boolean.
        '''
        if isinstance(expr, ResidueClass):
            if self.modulo == expr.modulo:
                if self.residue == expr.residue:
                    return True
        return False

    def __hash__(self):
        r'''Hashes residue class.

        Required to be explicitely re-defined on Python 3 if __eq__ changes.

        Returns integer.
        '''
        return super(ResidueClass, self).__hash__()

    def __lt__(self, expr):
        r'''Is true when `expr` is a residue class with module greater than that
        of this residue class. Also true when `expr` is a residue class with
        modulo equal to that of this residue class and with residue greater
        than that of this residue class. Otherwise false.

        Returns boolean.
        '''
        if isinstance(expr, ResidueClass):
            if self.modulo == expr.modulo:
                return self.residue < expr.residue
            return self.modulo < expr.modulo

    def __ne__(self, expr):
        r'''Is true when `expr` is not equal to this residue class. Otherwise
        false.

        Return boolean.
        '''
        return not self == expr

    ### PRIVATE PROPERTIES ###

    @property
    def _storage_format_specification(self):
        from abjad.tools import systemtools
        return systemtools.StorageFormatSpecification(
            self,
            is_indented=False,
            positional_argument_values=(
                self.modulo,
                self.residue,
                ),
            )

    ### PRIVATE METHODS ###

    def _initialize_by_modulo_and_residue(self, modulo, residue):
        if not 0 < modulo:
            message = 'modulo must be positive: {!r}.'
            message = message.format(modulo)
            raise ValueError(message)
        if not 0 <= residue < modulo:
            message = 'abs(residue) must be < modulo.'
            raise ValueError(message)
        self._modulo = modulo
        self._residue = residue

    def _initialize_by_rc_instance(self, rc):
        if not isinstance(rc, ResidueClass):
            message = 'must be residue class: {!r}.'
            message = message.format(rc)
            raise TypeError(message)
        self._modulo = rc.modulo
        self._residue = rc.residue

    ### PUBLIC PROPERTIES ###

    @property
    def modulo(self):
        r'''Period of residue class.
        '''
        return self._modulo

    @property
    def residue(self):
        r'''Residue of residue class.
        '''
        return self._residue

    ### PUBLIC METHODS ###

    def get_boolean_train(self, *min_max):
        r'''Returns a boolean train with 0s mapped to the integers
        that are not congruent bases of the residue class and 1s mapped
        to those that are.

        The method takes one or two integer arguments. If only one is given,
        it is taken as the max range and the min is assumed to be 0.

        ..  container:: example

            ::

                >>> r = RC(3, 0)
                >>> r.get_boolean_train(6)
                [1, 0, 0, 1, 0, 0]

            ::

                >>> r.get_congruent_bases(-6, 6)
                [-6, -3, 0, 3, 6]

        Returns list.
        '''
        minimum, maximum = self._process_min_max_attribute(*min_max)
        result = []
        for i in range(minimum, maximum):
            if i % self.modulo == self.residue:
                result.append(1)
            else:
                result.append(0)
        return result

    def get_congruent_bases(self, *min_max):
        r'''Returns all the congruent bases of this residue class within the
        given range.

        The method takes one or two integer arguments. If only one it given, it
        is taken as the max range and the min is assumed to be 0.

        ..  container:: example

            ::

                >>> r = RC(3, 0)
                >>> r.get_congruent_bases(6)
                [0, 3, 6]

            ::

                >>> r.get_congruent_bases(-6, 6)
                [-6, -3, 0, 3, 6]

        Returns list.
        '''
        minimum, maximum = self._process_min_max_attribute(*min_max)
        result = []
        for i in range(minimum, maximum + 1):
            if i % self.modulo == self.residue:
                result.append(i)
        return result


if __name__ == '__main__':
    print('Psappha B2[0:40]')
    RC = ResidueClass
    s1 = (RC(8, 0) | RC(8, 1) | RC(8, 7)) & (RC(5, 1) | RC(5, 3))
    s2 = (RC(8, 0) | RC(8, 1) | RC(8, 2)) & RC(5, 0)
    s3 = RC(8, 3) #&  RC(1, 0)
    s4 = RC(8, 4) #&  RC(1, 0)
    s5 = (RC(8, 5) | RC(8, 6)) & (RC(5, 2) | RC(5, 3) | RC(5, 4))
    s6 = (RC(8, 1) & RC(5, 2))
    s7 = (RC(8, 6) & RC(5, 1))

    y = s1 | s2 | s3 | s4 | s5 | s6 | s7
    print(y)
    print('congruent bases:\n', y.get_congruent_bases(40))
    print('boolen train:\n', y.get_boolean_train(40))


# -*- encoding: utf-8 -*-
from abjad import *


def test_labeltools_label_leaves_in_expr_with_leaf_durations_01():

    tuplet = scoretools.FixedDurationTuplet(Duration(2, 8), "c'8 d'8 e'8")
    labeltools.label_leaves_in_expr_with_leaf_durations(tuplet)

    assert systemtools.TestManager.compare(
        tuplet,
        r'''
        \times 2/3 {
            c'8
                _ \markup {
                    \column
                        {
                            \small
                                1/8
                            \small
                                1/12
                        }
                    }
            d'8
                _ \markup {
                    \column
                        {
                            \small
                                1/8
                            \small
                                1/12
                        }
                    }
            e'8
                _ \markup {
                    \column
                        {
                            \small
                                1/8
                            \small
                                1/12
                        }
                    }
        }
        '''
        )

    assert inspect_(tuplet).is_well_formed()


# -*- encoding: utf-8 -*-
import copy
from abjad.tools.datastructuretools.TypedList import TypedList


class PitchRangeInventory(TypedList):
    r'''An ordered list of pitch ranges.

    ::

        >>> inventory = pitchtools.PitchRangeInventory([
        ...     '[C3, C6]',
        ...     '[C4, C6]',
        ...     ])
        >>> inventory
        PitchRangeInventory([PitchRange(range_string='[C3, C6]'), PitchRange(range_string='[C4, C6]')])

    Pitch range inventories implement list interface and are mutable.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        )

    ### SPECIAL METHODS ###

    def __illustrate__(self):
        r'''Illustrates pitch range inventory.

        ::

            >>> show(inventory) # doctest: +SKIP

        Returns LilyPond file.
        '''
        from abjad.tools import durationtools
        from abjad.tools import lilypondfiletools
        from abjad.tools import indicatortools
        from abjad.tools import markuptools
        from abjad.tools import pitchtools
        from abjad.tools import scoretools
        from abjad.tools import spannertools
        from abjad.tools.topleveltools import attach
        from abjad.tools.topleveltools import iterate
        from abjad.tools.topleveltools import override
        start_note_clefs = []
        stop_note_clefs = []
        for pitch_range in self.items:
            start_note_clef = pitchtools.suggest_clef_for_named_pitches(
                pitch_range.start_pitch)
            start_note_clefs.append(start_note_clef)
            stop_note_clef = pitchtools.suggest_clef_for_named_pitches(
                pitch_range.stop_pitch)
            stop_note_clefs.append(stop_note_clef)
        if start_note_clefs == stop_note_clefs:
            clef = start_note_clefs[0]
            staff = scoretools.Staff()
            attach(clef, staff)
            score = scoretools.Score([staff])
            for pitch_range in self.items:
                start_note = scoretools.Note(pitch_range.start_pitch, 1)
                stop_note = scoretools.Note(pitch_range.stop_pitch, 1)
                notes = [start_note, stop_note]
                glissando = spannertools.Glissando()
                staff.extend(notes)
                attach(glissando, notes)
        else:
            result = scoretools.make_empty_piano_score()
            score, treble_staff, bass_staff = result
            for pitch_range in self.items:
                start_note = scoretools.Note(pitch_range.start_pitch, 1)
                start_note_clef = pitchtools.suggest_clef_for_named_pitches(
                    pitch_range.start_pitch)
                stop_note = scoretools.Note(pitch_range.stop_pitch, 1)
                stop_note_clef = pitchtools.suggest_clef_for_named_pitches(
                    pitch_range.stop_pitch)
                notes = [start_note, stop_note]
                glissando = spannertools.Glissando()
                skips = 2 * scoretools.Skip(1)
                treble_clef = indicatortools.Clef('treble')
                bass_clef = indicatortools.Clef('bass')
                if start_note_clef == stop_note_clef == treble_clef:
                    treble_staff.extend(notes)
                    bass_staff.extend(skips)
                elif start_note_clef == stop_note_clef == bass_clef:
                    bass_staff.extend(notes)
                    treble_staff.extend(skips)
                else:
                    assert start_note_clef == bass_clef
                    assert stop_note_clef == treble_clef
                    bass_staff.extend(notes)
                    treble_staff.extend(skips)
                    staff_change = indicatortools.StaffChange(treble_staff)
                    attach(staff_change, stop_note)
                attach(glissando, notes)
        for leaf in iterate(score).by_class(scoretools.Leaf):
            multiplier = durationtools.Multiplier(1, 4)
            attach(multiplier, leaf)
        override(score).bar_line.stencil = False
        override(score).span_bar.stencil = False
        override(score).glissando.thickness = 2
        override(score).time_signature.stencil = False
        lilypond_file = lilypondfiletools.make_basic_lilypond_file(score)
        lilypond_file.items.remove(lilypond_file['layout'])
        lilypond_file.items.remove(lilypond_file['paper'])
        lilypond_file.header_block.tagline = markuptools.Markup('""')
        return lilypond_file

    ### PRIVATE PROPERTIES ###

    @property
    def _attribute_manifest(self):
        from abjad.tools import systemtools
        return systemtools.AttributeManifest()

    @staticmethod
    def _item_callable(expr):
        from abjad.tools import pitchtools
        if isinstance(expr, str):
            range_ = pitchtools.PitchRange(expr)
        elif isinstance(expr, tuple):
            range_ = pitchtools.PitchRange.from_pitches(*expr)
        elif isinstance(expr, pitchtools.PitchRange):
            range_ = copy.copy(expr)
        else:
            raise TypeError(expr)
        return range_


# -*- encoding: utf-8 -*-


def extend_measures_in_expr_and_apply_full_measure_tuplets(expr, supplement):
    r'''Extend measures in `expr` with `supplement` and apply full-measure
    tuplets to contents of measures:

    ::

        >>> staff = Staff([Measure((2, 8), "c'8 d'8"), Measure((3, 8), "e'8 f'8 g'8")])

    ..  doctest::

        >>> print(format(staff))
        \new Staff {
            {
                \time 2/8
                c'8
                d'8
            }
            {
                \time 3/8
                e'8
                f'8
                g'8
            }
        }

    ::

        >>> supplement = [Rest((1, 16))]
        >>> scoretools.extend_measures_in_expr_and_apply_full_measure_tuplets(
        ... staff, supplement)

    ..  doctest::

        >>> print(format(staff))
        \new Staff {
            {
                \time 2/8
                \times 4/5 {
                    c'8
                    d'8
                    r16
                }
            }
            {
                \time 3/8
                \tweak #'text #tuplet-number::calc-fraction-text
                \times 6/7 {
                    e'8
                    f'8
                    g'8
                    r16
                }
            }
        }

    Returns none.
    '''
    from abjad.tools import scoretools

    return scoretools.apply_full_measure_tuplets_to_contents_of_measures_in_expr(
        expr, supplement)



# -*- encoding: utf-8 -*-
from abjad.tools import scoretools
from abjad.tools import markuptools
from abjad.tools.topleveltools import attach
from abjad.tools.topleveltools import iterate


def label_leaves_in_expr_with_leaf_depth(expr, markup_direction=Down):
    r'''Label leaves in `expr` with leaf depth:

    ::

        >>> staff = Staff("c'8 d'8 e'8 f'8 g'8")
        >>> scoretools.FixedDurationTuplet(Duration(2, 8), staff[-3:])
        FixedDurationTuplet(Duration(1, 4), "e'8 f'8 g'8")
        >>> labeltools.label_leaves_in_expr_with_leaf_depth(staff)
        >>> show(staff) # doctest: +SKIP

    ..  doctest::

        >>> print(format(staff))
        \new Staff {
            c'8 _ \markup { \small 1 }
            d'8 _ \markup { \small 1 }
            \times 2/3 {
                e'8 _ \markup { \small 2 }
                f'8 _ \markup { \small 2 }
                g'8 _ \markup { \small 2 }
            }
        }

    ::

        >>> show(staff) # doctest: +SKIP

    Returns none.
    '''

    for leaf in iterate(expr).by_class(scoretools.Leaf):
        label = markuptools.MarkupCommand(
            'small', str(leaf._get_parentage().depth))
        markup = markuptools.Markup(label, markup_direction)
        attach(markup, leaf)



# -*- encoding: utf-8 -*-
import abc
import collections
import types
from abjad.tools import datastructuretools
from abjad.tools.datastructuretools import TypedCounter
from abjad.tools.topleveltools import new


class Vector(TypedCounter):
    '''Music-theoretic vector base class.
    '''

    ### CLASS VARIABLES ##

    __slots__ = (
        )

    ### INITIALIZER ###

    def __init__(self, items=None, item_class=None):
        from abjad.tools import datastructuretools
        from abjad.tools import pitchtools
        if isinstance(items, str):
            items = items.split()
        elif isinstance(items, (
            collections.Iterator,
            types.GeneratorType,
            )):
            items = [item for item in items]
        if isinstance(items, (TypedCounter, collections.Counter)):
            new_tokens = []
            for item, count in items.items():
                new_tokens.extend(count * [item])
            items = new_tokens
        if item_class is None:
            item_class = self._named_item_class
            if items is not None:
                if isinstance(items, datastructuretools.TypedCollection) and \
                    issubclass(items.item_class, self._parent_item_class):
                    item_class = items.item_class
                elif len(items):
                    if isinstance(items, collections.Set):
                        items = tuple(items)
                    if isinstance(items[0], str):
                        item_class = self._named_item_class
                    elif isinstance(items[0], (int, float)):
                        item_class = self._numbered_item_class
                    elif isinstance(items[0], self._parent_item_class):
                        item_class = type(items[0])
        assert issubclass(item_class, self._parent_item_class)
        TypedCounter.__init__(
            self,
            items=items,
            item_class=item_class,
            )

    ### SPECIAL METHODS ###

    def __str__(self):
        r'''String representation of vector.

        Returns string.
        '''
        parts = ['{}: {}'.format(key, value)
            for key, value in self.items()]
        return '<{}>'.format(', '.join(parts))

    ### PRIVATE PROPERTIES ###

    @abc.abstractproperty
    def _named_item_class(self):
        raise NotImplementedError

    @abc.abstractproperty
    def _numbered_item_class(self):
        raise NotImplementedError

    @abc.abstractproperty
    def _parent_item_class(self):
        raise NotImplementedError

    @property
    def _repr_specification(self):
        items = {}
        for key, value in self:
            items[str(key)] = value
        return new(
            self._storage_format_specification,
            is_indented=False,
            keyword_argument_names=(),
            positional_argument_values=(
                items,
                ),
            )

    ### PUBLIC METHODS ###

    @abc.abstractmethod
    def from_selection(
        cls,
        selection,
        item_class=None,
        ):
        r'''Makes vector from `selection`.

        Returns vector.
        '''
        raise NotImplementedError


# -*- encoding: utf-8 -*-
from abjad import *
from abjad.demos.ferneyhough.configure_score import configure_score
from abjad.demos.ferneyhough.configure_lilypond_file import configure_lilypond_file
from abjad.demos.ferneyhough.make_score import make_score


def make_lilypond_file(tuplet_duration, row_count, column_count):
    r'''Makes LilyPond file.
    '''

    score = make_score(tuplet_duration, row_count, column_count)
    configure_score(score)
    lilypond_file = lilypondfiletools.make_basic_lilypond_file(score)
    configure_lilypond_file(lilypond_file)
    return lilypond_file


# -*- encoding: utf-8 -*-
import os
from abjad.tools import documentationtools
from abjad.tools.developerscripttools.DeveloperScript import DeveloperScript


class MakeNewClassTemplateScript(DeveloperScript):
    r'''Creates class stubs and test subdirectory.

    ..  shell::

        ajv new class --help

    '''

    ### PUBLIC PROPERTIES ###

    @property
    def alias(self):
        r'''Alias of script.

        Returns ``'class'``.
        '''
        return 'class'

    @property
    def long_description(self):
        r'''Long description of script.

        Returns string or none.
        '''
        return None

    @property
    def scripting_group(self):
        r'''Scripting group of script.

        Returns ``'new'``.
        '''
        return 'new'

    @property
    def short_description(self):
        r'''Short description of script.

        Returns string.
        '''
        return 'Make a new class template file.'

    @property
    def version(self):
        r'''Version of script.

        Returns float.
        '''
        return 1.0

    ### PRIVATE METHODS ###

    def _get_class_names_in_tools_package(self, root, tools_package_name):
        path = os.path.join(root, tools_package_name)
        crawler = documentationtools.ClassCrawler(
            path, include_private_objects=True)
        objs = crawler()
        return tuple(sorted([x.__name__ for x in objs]))

    def _get_class_text(self, class_name):
        return [
            '# -*- encoding: utf-8 -*-',
            'from abjad.tools.abctools import AbjadObject',
            '',
            '',
            'class {}(AbjadObject):'.format(class_name),
            '    pass'
            ]

    def _get_tools_package_names(self, root):
        names = []
        for x in os.listdir(root):
            if os.path.isdir(os.path.join(root, x)):
                if not x.startswith(('_', '.')):
                    names.append(x)
        return tuple(sorted(names))

    ### PUBLIC METHODS ###

    def process_args(self, args):
        r'''Processes `args`.

        Returns none.
        '''
        from abjad import abjad_configuration
        if args.name.count('.') != 1:
            message = 'not in tools_package.class_name format: {!r}.'
            message = message.format(args.name)
            raise SystemExit(message)
        root_directory = args.path
        tools_package_name, class_name = args.name.split('.')
        if tools_package_name not in self._get_tools_package_names(
            root_directory):
            message = '{!r} is not a valid tools package.'
            message = message.format(tools_package_name)
            raise SystemExit(message)
        if class_name in self._get_class_names_in_tools_package(
            root_directory, tools_package_name):
            message = '{!r} already exists in {!r}'
            message = message.format(class_name, tools_package_name)
            raise SystemExit(message)
        tools_package_directory = os.path.join(
            root_directory,
            tools_package_name,
            )
        if args.path == os.path.join(
            abjad_configuration.abjad_experimental_directory, 'tools'):
            package_root_name = 'experimental'
        elif args.path == os.path.join(
            abjad_configuration.abjad_directory, 'tools'):
            package_root_name = 'abjad'
        else:
            raise SystemExit
        module_file_path = os.path.join(
            tools_package_directory,
            '{}.py'.format(class_name),
            )
        module_text = '\n'.join(self._get_class_text(class_name)) + '\n'
        with open(module_file_path, 'w') as f:
            f.write(module_text)

    def setup_argument_parser(self, parser):
        r'''Sets up argument `parser`.

        Returns none.
        '''

        from abjad import abjad_configuration
        parser.add_argument('name',
            help='tools package qualified class name'
            )
        path_group = parser.add_mutually_exclusive_group(required=True)
        path_group.add_argument('-X', '--experimental',
            action='store_const',
            const=os.path.join(
                abjad_configuration.abjad_experimental_directory,
                'tools'),
            dest='path',
            help='use the Abjad experimental tools path',
            )
        path_group.add_argument('-M', '--mainline',
            action='store_const',
            const=os.path.join(
                abjad_configuration.abjad_directory, 'tools'),
            dest='path',
            help='use the Abjad mainline tools path',
            )


# -*- encoding: utf-8 -*-


def all_are_positive_integer_equivalent_numbers(expr):
    '''Is true when `expr` is a sequence and all elements in `expr` are
    positive integer-equivalent numbers. Otherwise false:

    ::

        >>> mathtools.all_are_positive_integer_equivalent_numbers([Fraction(4, 2), 2.0, 2])
        True

    Returns boolean.
    '''
    from abjad.tools import mathtools

    try:
        return all(mathtools.is_positive_integer_equivalent_number(x) for x in expr)
    except TypeError:
        return False


# -*- encoding: utf-8 -*-
from abjad.tools.abctools.AbjadObject import AbjadObject


class BarLine(AbjadObject):
    r'''A bar line.

    ::

        >>> staff = Staff("c'4 d'4 e'4 f'4")
        >>> bar_line = indicatortools.BarLine('|.')
        >>> attach(bar_line, staff[-1])
        >>> show(staff) # doctest: +SKIP

    ::

        >>> bar_line
        BarLine('|.')

    ..  doctest::

        >>> print(format(staff))
        \new Staff {
            c'4
            d'4
            e'4
            f'4
            \bar "|."
        }

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_abbreviation',
        '_default_scope',
        )

    _format_slot = 'closing'

    ### INITIALIZER ##

    def __init__(self, abbreviation='|'):
        from abjad.tools import scoretools
        assert isinstance(abbreviation, str), repr(abbreviation)
        self._abbreviation = abbreviation
        self._default_scope = scoretools.Staff

    ### SPECIAL METHODS ###

    def __copy__(self, *args):
        r'''Copies bar line.

        Returns new bar line.
        '''
        return type(self)(self.abbreviation)

    def __eq__(self, arg):
        r'''Is true when `arg` is a bar line with an abbreviation equal
        to that of this bar line. Otherwise false.

        Returns boolean.
        '''
        if isinstance(arg, type(self)):
            return self.abbreviation == arg.abbreviation
        return False

    def __hash__(self):
        r'''Hashes bar line.

        Required to be explicitely re-defined on Python 3 if __eq__ changes.

        Returns integer.
        '''
        return super(BarLine, self).__hash__()

    ### PRIVATE PROPERTIES ###

    @property
    def _contents_repr_string(self):
        return repr(self.abbreviation)

    @property
    def _lilypond_format(self):
        return r'\bar "{}"'.format(self.abbreviation)

    @property
    def _repr_specification(self):
        from abjad.tools import systemtools
        return systemtools.StorageFormatSpecification(
            self,
            is_indented=False,
            keyword_argument_names=(),
            positional_argument_values=(
                self.abbreviation,
                ),
            )

    ## PUBLIC PROPERTIES ##

    @property
    def abbreviation(self):
        r'''Abbreviation of bar line.

        ::

            >>> bar_line.abbreviation
            '|.'

        Returns string.
        '''
        return self._abbreviation



# -*- encoding: utf-8 -*-


def is_nonnegative_integer_equivalent_number(expr):
    '''Is true when `expr` is a nonnegative integer-equivalent number.
    Otherwise false:

    ::

        >>> mathtools.is_nonnegative_integer_equivalent_number(Duration(4, 2))
        True

    Returns boolean.
    '''
    from abjad.tools import mathtools

    return mathtools.is_integer_equivalent_number(expr) and 0 <= expr


# -*- encoding: utf-8 -*-
from abjad.tools.spannertools.Spanner import Spanner
from abjad.tools.topleveltools import override


class TextSpanner(Spanner):
    r'''A text spanner.

    ..  container:: example

        ::

            >>> staff = Staff("c'8 d'8 e'8 f'8")
            >>> text_spanner = spannertools.TextSpanner()
            >>> grob = override(text_spanner).text_spanner
            >>> markup_command = markuptools.MarkupCommand('italic', 'foo')
            >>> markup_command = markuptools.MarkupCommand('bold', markup_command)
            >>> left_markup = markuptools.Markup(markup_command)
            >>> grob.bound_details__left__text = left_markup
            >>> pair = schemetools.SchemePair(0, -1)
            >>> markup_command = markuptools.MarkupCommand('draw-line', pair)
            >>> right_markup = markuptools.Markup(markup_command)
            >>> grob.bound_details__right__text = right_markup
            >>> override(text_spanner).text_spanner.dash_fraction = 1
            >>> attach(text_spanner, [staff])
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> print(format(staff))
            \new Staff {
                \override TextSpanner #'bound-details #'left #'text = \markup {
                    \bold \italic foo }
                \override TextSpanner #'bound-details #'right #'text = \markup {
                    \draw-line #'(0 . -1) }
                \override TextSpanner #'dash-fraction = #1
                c'8 \startTextSpan
                d'8
                e'8
                f'8 \stopTextSpan
                \revert TextSpanner #'bound-details
                \revert TextSpanner #'dash-fraction
            }

    Formats LilyPond ``\startTextSpan`` command on first leaf in spanner.

    Formats LilyPond ``\stopTextSpan`` command on last leaf in spanner.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_dynamic_text',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        overrides=None,
        ):
        Spanner.__init__(
            self,
            overrides=overrides,
            )

    ### PRIVATE METHODS ###

    def _get_lilypond_format_bundle(self, leaf):
        from abjad.tools import systemtools
        lilypond_format_bundle = systemtools.LilyPondFormatBundle()
        if self._is_my_first_leaf(leaf):
            contributions = override(self)._list_format_contributions(
                'override',
                is_once=False,
                )
            lilypond_format_bundle.grob_overrides.extend(contributions)
            string = r'\startTextSpan'
            lilypond_format_bundle.right.spanner_starts.append(string)
        if self._is_my_last_leaf(leaf):
            contributions = override(self)._list_format_contributions(
                'revert',
                )
            lilypond_format_bundle.grob_reverts.extend(contributions)
            string = r'\stopTextSpan'
            lilypond_format_bundle.right.spanner_stops.append(string)
        return lilypond_format_bundle



# -*-encoding: utf-8 -*-
import copy
import types
from abjad.tools import durationtools
from abjad.tools import scoretools
from abjad.tools import selectiontools
from abjad.tools import timespantools
from abjad.tools.abctools import AbjadObject
from abjad.tools.topleveltools import set_
from abjad.tools.topleveltools import iterate
from abjad.tools.topleveltools import override
Selection = selectiontools.Selection


class Spanner(AbjadObject):
    '''Any type of object that stretches horizontally
    and encompasses some number of score components.

    Examples include beams, slurs, hairpins and trills.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_components',
        '_contiguity_constraint',
        '_indicator_expressions',
        '_lilypond_grob_name_manager',
        '_lilypond_setting_name_manager',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        overrides=None,
        ):
        overrides = overrides or {}
        self._components = []
        self._contiguity_constraint = 'logical voice'
        self._apply_overrides(overrides)
        self._indicator_expressions = []
        self._lilypond_setting_name_manager = None

    ### SPECIAL METHODS ###

    def __contains__(self, expr):
        r'''Is true when spanner contains `expr`.
        Otherwise false.

        Returns boolean.
        '''
        for x in self._components:
            if x is expr:
                return True
        else:
            return False

    def __copy__(self, *args):
        r'''Copies spanner.

        Does not copy spanner components.

        Returns new spanner.
        '''
        new = type(self)(*self.__getnewargs__())
        if getattr(self, '_lilypond_grob_name_manager', None) is not None:
            new._lilypond_grob_name_manager = copy.copy(override(self))
        if getattr(self, '_lilypond_setting_name_manager', None) is not None:
            new._lilypond_setting_name_manager = copy.copy(set_(self))
        self._copy_keyword_args(new)
        return new

    def __getitem__(self, expr):
        r'''Gets item from spanner.

        Returns component.
        '''
        return self._components.__getitem__(expr)

    def __getnewargs__(self):
        r'''Gets new arguments of spanner.

        Returns empty tuple.
        '''
        return ()

    def __getstate__(self):
        r'''Gets state of spanner.

        Returns dictionary.
        '''
        state = {}
        for class_ in type(self).__mro__:
            for slot in getattr(class_, '__slots__', ()):
                state[slot] = getattr(self, slot, None)
        return state

    def __len__(self):
        r'''Gets number of components in spanner.

        Returns nonnegative integer.
        '''
        return len(self.components)

    def __lt__(self, expr):
        r'''Is true when spanner is less than `expr`. Otherwise false.

        Trivial comparison to allow doctests to work.

        Returns boolean.
        '''
        if not isinstance(expr, Spanner):
            raise TypeError
        return repr(self) < repr(expr)

    ### PRIVATE PROPERTIES ###

    @property
    def _compact_summary(self):
        len_self = len(self)
        if not len_self:
            return ''
        elif 0 < len_self <= 8:
            return ', '.join([str(x) for x in self])
        else:
            left = ', '.join([str(x) for x in self[:2]])
            right = ', '.join([str(x) for x in self[-2:]])
            number_in_middle = len_self - 4
            middle = ', ... [%s] ..., ' % number_in_middle
            return left + middle + right

    @property
    def _duration_in_seconds(self):
        duration = durationtools.Duration(0)
        for leaf in self.leaves:
            duration += leaf._get_duration(in_seconds=True)
        return duration

    @property
    def _leaves(self):
        result = []
        for component in self._components:
            for node in iterate(component).depth_first():
                if isinstance(node, scoretools.Leaf):
                    result.append(node)
        result = tuple(result)
        return result

    @property
    def _preprolated_duration(self):
        return sum([component._preprolated_duration for component in self])

    @property
    def _repr_specification(self):
        from abjad.tools import systemtools
        return systemtools.StorageFormatSpecification(
            self,
            is_indented=False,
            keyword_argument_names=(),
            positional_argument_values=(
                self._compact_summary,
                )
            )

    @property
    def _storage_format_specification(self):
        from abjad.tools import systemtools
        keyword_argument_names = list(
            systemtools.StorageFormatManager.get_keyword_argument_names(self))
        if 'overrides' in keyword_argument_names:
            keyword_argument_names.remove('overrides')
        return systemtools.StorageFormatSpecification(
            self,
            keyword_argument_names=keyword_argument_names,
            )

    @property
    def _summary(self):
        if 0 < len(self):
            return ', '.join([str(x) for x in self])
        else:
            return ' '

    ### PRIVATE METHODS ###

    def _append(self, component):
        if self._contiguity_constraint == 'logical voice':
            components = self[-1:] + [component]
            assert Selection._all_are_contiguous_components_in_same_logical_voice(
                components), repr(components)
        component._spanners.add(self)
        self._components.append(component)

    def _append_left(self, component):
        components = [component] + self[:1]
        assert Selection._all_are_contiguous_components_in_same_logical_voice(
            components)
        component._spanners.add(self)
        self._components.insert(0, component)

    def _apply_overrides(self, overrides):
        exec('from abjad import *')
        for grob_attribute_string in overrides:
            grob_value_string = overrides[grob_attribute_string]
            statement = 'override(self).{} = {}'
            grob_attribute_string = grob_attribute_string.replace('__', '.', 1)
            grob_value_string = grob_value_string.replace('\t', '')
            strings = (grob_attribute_string, grob_value_string)
            statement = statement.format(*strings)
            exec(statement)

    def _attach(self, components):
        from abjad.tools import scoretools
        from abjad.tools import selectiontools
        assert not self, repr(self)
        if isinstance(components, scoretools.Component):
            self._append(components)
        elif isinstance(components, (list, tuple, selectiontools.Selection)):
            self._extend(components)
        else:
            raise TypeError(components)

    def _block_all_components(self):
        r'''Not composer-safe.
        '''
        for component in self:
            self._block_component(component)

    def _block_component(self, component):
        r'''Not composer-safe.
        '''
        component._spanners.remove(self)

    def _copy(self, components):
        r'''Returns copy of spanner with `components`.
        `components` must be an iterable of components already
        contained in spanner.
        '''
        my_components = self._components[:]
        self._components = []
        result = copy.copy(self)
        self._components = my_components
        for component in components:
            assert component in self
        for component in components:
            result._components.append(component)
        result._unblock_all_components()
        return result

    def _copy_keyword_args(self, new):
        pass

    def _detach(self):
        self._sever_all_components()

    def _extend(self, components):
        component_input = self[-1:]
        component_input.extend(components)
        if self._contiguity_constraint == 'logical voice':
            assert Selection._all_are_contiguous_components_in_same_logical_voice(
                component_input), repr(component_input)
        for component in components:
            self._append(component)

    def _extend_left(self, components):
        component_input = components + self[:1]
        assert Selection._all_are_contiguous_components_in_same_logical_voice(
            component_input)
        for component in reversed(components):
            self._append_left(component)

    def _format_after_leaf(self, leaf):
        result = []
        if self._is_my_last_leaf(leaf):
            result.extend(getattr(self, '_reverts', []))
        return result

    def _format_before_leaf(self, leaf):
        result = []
        if self._is_my_first_leaf(leaf):
            result.extend(getattr(self, '_overrides', []))
        return result

    def _format_right_of_leaf(self, leaf):
        result = []
        return result

    def _fracture(self, i, direction=None):
        r'''Fractures spanner at `direction` of component at index `i`.

        Valid values for `direction` are ``Left``, ``Right`` and ``None``.

        Set `direction=None` to fracture on both left and right sides.

        Returns tuple of original, left and right spanners.
        '''
        if i < 0:
            i = len(self) + i
        if direction == Left:
            return self._fracture_left(i)
        elif direction == Right:
            return self._fracture_right(i)
        elif direction is None:
            left = self._copy(self[:i])
            right = self._copy(self[i+1:])
            center = self._copy(self[i:i+1])
            self._block_all_components()
            return self, left, center, right
        else:
            message = 'direction {!r} must be left, right or none.'
            message = message.format(direction)
            raise ValueError(message)

    def _fracture_left(self, i):
        left = self._copy(self[:i])
        right = self._copy(self[i:])
        self._block_all_components()
        return self, left, right

    def _fracture_right(self, i):
        left = self._copy(self[:i + 1])
        right = self._copy(self[i + 1:])
        self._block_all_components()
        return self, left, right

    def _fuse_by_reference(self, spanner):
        result = self._copy(self[:])
        result._extend(spanner.components)
        self._block_all_components()
        spanner._block_all_components()
        return [(self, spanner, result)]

    def _get_basic_lilypond_format_bundle(self, leaf):
        from abjad.tools import systemtools
        lilypond_format_bundle = systemtools.LilyPondFormatBundle()
        if self._is_my_first_leaf(leaf):
            contributions = override(self)._list_format_contributions(
                'override',
                is_once=False,
                )
            lilypond_format_bundle.grob_overrides.extend(contributions)
        if self._is_my_last_leaf(leaf):
            contributions = override(self)._list_format_contributions(
                'revert',
                )
            lilypond_format_bundle.grob_reverts.extend(contributions)
        return lilypond_format_bundle

    def _get_duration(self, in_seconds=False):
        return sum(
            component._get_duration(in_seconds=in_seconds)
            for component in self
            )

    def _get_indicators(self, prototype=None, unwrap=True):
        from abjad.tools import indicatortools
        prototype = prototype or (object,)
        if not isinstance(prototype, tuple):
            prototype = (prototype,)
        prototype_objects, prototype_classes = [], []
        for indicator_prototype in prototype:
            if isinstance(indicator_prototype, type):
                prototype_classes.append(indicator_prototype)
            else:
                prototype_objects.append(indicator_prototype)
        prototype_objects = tuple(prototype_objects)
        prototype_classes = tuple(prototype_classes)
        matching_indicators = []
        for indicator in self._indicator_expressions:
            if isinstance(indicator, prototype_classes):
                matching_indicators.append(indicator)
            elif any(indicator == x for x in prototype_objects):
                matching_indicators.append(indicator)
            elif isinstance(indicator, indicatortools.IndicatorExpression):
                if isinstance(indicator.indicator, prototype_classes):
                    matching_indicators.append(indicator)
                elif any(indicator.indicator == x for x in prototype_objects):
                    matching_indicators.append(indicator)
        if unwrap:
            matching_indicators = [x.indicator for x in matching_indicators]
        matching_indicators = tuple(matching_indicators)
        return matching_indicators

    def _get_lilypond_format_bundle(self, leaf):
        lilypond_format_bundle = self._get_basic_lilypond_format_bundle(leaf)
        lilypond_format_bundle.get('before').spanners.extend(
            self._format_before_leaf(leaf))
        lilypond_format_bundle.get('right').spanners.extend(
            self._format_right_of_leaf(leaf))
        lilypond_format_bundle.get('after').spanners.extend(
            self._format_after_leaf(leaf))
        return lilypond_format_bundle

    def _get_my_first_leaf(self):
        for leaf in iterate(self).by_class(scoretools.Leaf):
            return leaf

    def _get_my_last_leaf(self):
        for leaf in iterate(self).by_class(scoretools.Leaf, reverse=True):
            return leaf

    def _get_my_nth_leaf(self, n):
        from abjad.tools import scoretools
        if not isinstance(n, int):
            raise TypeError
        if 0 <= n:
            leaves = iterate(self).by_class(scoretools.Leaf)
            for leaf_index, leaf in enumerate(leaves):
                if leaf_index == n:
                    return leaf
        else:
            leaves = iterate(self).by_class(scoretools.Leaf, reverse=True)
            for leaf_index, leaf in enumerate(leaves):
                leaf_number = -leaf_index - 1
                if leaf_number == n:
                    return leaf
        raise IndexError

    def _get_timespan(self, in_seconds=False):
        if len(self):
            start_offset = \
                self[0]._get_timespan(in_seconds=in_seconds).start_offset
        else:
            start_offset = Duration(0)
        if len(self):
            stop_offset = \
                self[-1]._get_timespan(in_seconds=in_seconds).stop_offset
        else:
            stop_offset = Duration(0)
        return timespantools.Timespan(
            start_offset=start_offset, stop_offset=stop_offset)

    def _index(self, component):
        return self._components.index(component)

    def _insert(self, i, component):
        r'''Not composer-safe.
        '''
        component._spanners.add(self)
        self._components.insert(i, component)

    def _is_exterior_leaf(self, leaf):
        r'''True if leaf is first or last in spanner.
        True if next leaf or previous leaf is none.
        Otherwise false.
        '''
        if self._is_my_first_leaf(leaf):
            return True
        elif self._is_my_last_leaf(leaf):
            return True
        elif not leaf._get_leaf(-1) and not leaf._get_leaf(1):
            return True
        else:
            return False

    def _is_interior_leaf(self, leaf):
        leaves = self._leaves
        if leaf not in leaves:
            return False
        if len(leaves) < 3:
            return False
        leaf_count = len(leaves)
        first_index = 0
        last_index = leaf_count - 1
        leaf_index = leaves.index(leaf)
        if first_index < leaf_index < last_index:
            return True
        return False

    def _is_my_first(self, leaf, prototype):
        for component in iterate(self).by_class(prototype):
            if component is leaf:
                return True
            else:
                return False

    def _is_my_first_leaf(self, leaf):
        try:
            first_leaf = self._get_my_nth_leaf(0)
            return leaf is first_leaf
        except IndexError:
            return False

    def _is_my_last(self, leaf, prototype):
        for component in iterate(self).by_class(
            prototype,
            reverse=True,
            ):
            if component is leaf:
                return True
            else:
                return False

    def _is_my_last_leaf(self, leaf):
        try:
            last_leaf = self._get_my_nth_leaf(-1)
            return leaf is last_leaf
        except IndexError:
            return False

    def _is_my_only(self, leaf, prototype):
        i = None
        components = iterate(self).by_class(prototype)
        for i, component in enumerate(components):
            if 0 < i:
                return False
        return i == 0

    def _is_my_only_leaf(self, leaf):
        return self._is_my_first_leaf(leaf) and self._is_my_last_leaf(leaf)

    def _make_storage_format_with_overrides(self):
        from abjad.tools import systemtools
        override_dictionary = override(self)._make_override_dictionary()
        lines = []
        line = '{}.{}('.format(
            systemtools.StorageFormatManager.get_tools_package_name(self),
            type(self).__name__,
            )
        lines.append(line)
        lines.append('\toverrides = {')
        for key, value in override_dictionary.items():
            value = value.replace('\t', '')
            line = '\t\t{!r}: {!r},'.format(key, value)
            lines.append(line)
        lines.append('\t}')
        lines.append(')')
        result = '\n'.join(lines)
        return result

    def _remove(self, component):
        r'''Not composer-safe.
        '''
        self._sever_component(component)

    def _remove_component(self, component):
        r'''Not composer-safe.
        '''
        for i, x in enumerate(self._components):
            if x is component:
                self._components.pop(i)
                break
        else:
            message = 'component {!r} not in spanner components list.'
            raise ValueError(message.format(component))

    def _reverse_components(self):
        r'''Reverses order of spanner components.

        Not composer-safe because reversing the order of spanner components
        could scramble components of some other spanner.

        Call method only as part of a full component- and spanner-reversal
        routine.

        Spanner subclasses with mapping variables (like the 'durations' list
        attaching to durated complex beam spanners) should override this
        method to reverse mapping elements.
        '''
        self._components.reverse()

    def _sever_all_components(self):
        r'''Not composer-safe.
        '''
        for n in reversed(range(len(self))):
            component = self[n]
            self._sever_component(component)

    def _sever_component(self, component):
        r'''Not composer-safe.
        '''
        self._block_component(component)
        self._remove_component(component)

    def _start_offset_in_me(self, leaf):
        leaf_start_offset = leaf._get_timespan().start_offset
        self_start_offset = self._get_timespan().start_offset
        return leaf_start_offset - self_start_offset

    def _stop_offset_in_me(self, leaf):
        leaf_start_offset = self._start_offset_in_me(leaf)
        leaf_stop_offset = leaf_start_offset + leaf._get_duration()
        return leaf_stop_offset

    def _unblock_all_components(self):
        r'''Not composer-safe.
        '''
        for component in self:
            self._unblock_component(component)

    def _unblock_component(self, component):
        r'''Not composer-safe.
        '''
        component._spanners.add(self)

    ### PUBLIC PROPERTIES ###

    @property
    def components(self):
        r'''Selects components in spanner.

        Returns selection.
        '''
        from abjad.tools import selectiontools
        return selectiontools.Selection(self._components[:])


# -*- encoding: utf-8 -*-
import pytest
from abjad import *


def test_sequencetools_flatten_sequence_01():

    l = [1, 2, 3, 4, 5]
    new = sequencetools.flatten_sequence(l)
    assert new == [1, 2, 3, 4, 5]


def test_sequencetools_flatten_sequence_02():

    l = [(1, 2), [3, 4]]
    new = sequencetools.flatten_sequence(l)
    assert new == [1, 2, 3, 4]


def test_sequencetools_flatten_sequence_03():

    l = [(1, 2), [3, (4, 5)]]
    new = sequencetools.flatten_sequence(l)
    assert new == [1, 2, 3, 4, 5]


def test_sequencetools_flatten_sequence_04():

    l = [(1, 2), [3, (4, 5)]]
    new = sequencetools.flatten_sequence(l, classes=(list, ))
    assert new == [(1, 2), 3, (4, 5)]


def test_sequencetools_flatten_sequence_05():

    l = [(1, 2), [3, (4, 5)]]
    statement = 'sequencetools.flatten_sequence(l, classes=(tuple, ))'
    assert pytest.raises(AssertionError, statement)


def test_sequencetools_flatten_sequence_06():

    l = [1, [2, 3, [4]], 5, [6, 7, [8]]]
    assert sequencetools.flatten_sequence(l, depth=0) == [1, [2, 3, [4]], 5, [6, 7, [8]]]
    assert sequencetools.flatten_sequence(l, depth=1) == [1, 2, 3, [4], 5, 6, 7, [8]]
    assert sequencetools.flatten_sequence(l, depth=2) == [1, 2, 3, 4, 5, 6, 7, 8]
    assert sequencetools.flatten_sequence(l, depth=2) == sequencetools.flatten_sequence(l, depth=99)


def test_sequencetools_flatten_sequence_07():
    r'''Works with positive indices.
    '''

    sequence_1 = [0, 1, [2, 3, 4], [5, 6, 7]]
    sequence_2 = sequencetools.flatten_sequence(sequence_1, indices=[2])

    assert sequence_2 == [0, 1, 2, 3, 4, [5, 6, 7]]


def test_sequencetools_flatten_sequence_08():
    r'''Works with negative indices.
    '''

    sequence_1 = [0, 1, [2, 3, 4], [5, 6, 7]]
    works = sequencetools.flatten_sequence(sequence_1, indices=[-1])

    assert works == [0, 1, [2, 3, 4], 5, 6, 7]


def test_sequencetools_flatten_sequence_09():
    r'''Boundary cases.
    '''

    sequence_1 = [0, 1, [2, 3, 4], [5, 6, 7]]

    sequence_2 = sequencetools.flatten_sequence(sequence_1, indices=[])
    assert sequence_2 == [0, 1, [2, 3, 4], [5, 6, 7]]

    sequence_2 = sequencetools.flatten_sequence(sequence_1, indices=[99])
    assert sequence_2 == [0, 1, [2, 3, 4], [5, 6, 7]]

    sequence_2 = sequencetools.flatten_sequence(sequence_1, indices=[-99])
    assert sequence_2 == [0, 1, [2, 3, 4], [5, 6, 7]]


# -*- encoding: utf-8 -*-
from abjad import *
from abjad.tools.pitchtools import OctaveTranspositionMapping
from abjad.tools.pitchtools import OctaveTranspositionMappingInventory


def test_pitchtools_OctaveTranspositionMappingInventory___repr___01():

    inventory_1 = pitchtools.OctaveTranspositionMappingInventory([[('[A0, C8]', -18)]])
    inventory_2 = eval(repr(inventory_1))

    assert isinstance(inventory_1, pitchtools.OctaveTranspositionMappingInventory)
    assert isinstance(inventory_2, pitchtools.OctaveTranspositionMappingInventory)
    assert inventory_1 == inventory_2


# -*- encoding: utf-8 -*-
import inspect
import pytest
import abjad
from abjad.tools import documentationtools
from abjad.tools import metertools
from abjad.tools import tonalanalysistools


classes = documentationtools.list_all_abjad_classes()
@pytest.mark.parametrize('class_', classes)
def test_abjad___format___01(class_):
    r'''All concrete classes have a storage format.
    '''

    if '_storage_format_specification' in dir(class_) and \
        not inspect.isabstract(class_):
        instance = class_()
        instance_format = format(instance, 'storage')
        assert isinstance(instance_format, str)
        assert not instance_format == ''


# TODO: make these work eventually
_classes_to_temporarily_skip = (
    metertools.Meter,
    tonalanalysistools.RootedChordClass,
    )

classes = documentationtools.list_all_abjad_classes()
@pytest.mark.parametrize('class_', classes)
def test_abjad___format___02(class_):
    r'''All storage-formattable classes have evaluable storage format.
    '''

    if '_storage_format_specification' in dir(class_) and \
        not inspect.isabstract(class_) and \
        class_ not in _classes_to_temporarily_skip:
        environment = abjad.__dict__.copy()
        environment.update(abjad.demos.__dict__)
        instance_one = class_()
        instance_one_format = format(instance_one, 'storage')
        assert isinstance(instance_one_format, str)
        assert instance_one_format != ''
        instance_two = eval(instance_one_format, environment)
        instance_two_format = format(instance_two, 'storage')
        assert instance_one_format == instance_two_format


# -*- encoding: utf-8 -*-
from abjad.tools import durationtools
from abjad.tools import indicatortools
from abjad.tools import mathtools
from abjad.tools import scoretools
from abjad.tools import selectiontools
from abjad.tools import sequencetools
from abjad.tools import spannertools
from abjad.tools import systemtools
from abjad.tools.rhythmmakertools.BeamSpecifier import BeamSpecifier
from abjad.tools.rhythmmakertools.ExampleWrapper import ExampleWrapper
from abjad.tools.rhythmmakertools.RhythmMaker import RhythmMaker
from abjad.tools.rhythmmakertools.TieSpecifier import TieSpecifier
from abjad.tools.topleveltools import attach
from abjad.tools.topleveltools import iterate
from abjad.tools.topleveltools import new


class EvenRunRhythmMaker(RhythmMaker):
    r'''Even run rhythm-maker.

    ..  container:: example

        Makes even run of notes each equal in duration to ``1/d``
        with ``d`` equal to the denominator of each division on which
        the rhythm-maker is called:

        ::

            >>> maker = rhythmmakertools.EvenRunRhythmMaker()

        ::

            >>> divisions = [(4, 8), (3, 4), (2, 4)]
            >>> music = maker(divisions)
            >>> lilypond_file = rhythmmakertools.make_lilypond_file(
            ...     music,
            ...     divisions,
            ...     )
            >>> show(lilypond_file) # doctest: +SKIP

        ..  doctest::

            >>> staff = maker._get_rhythmic_staff(lilypond_file)
            >>> f(staff)
            \new RhythmicStaff {
                {
                    \time 4/8
                    {
                        c'8 [
                        c'8
                        c'8
                        c'8 ]
                    }
                }
                {
                    \time 3/4
                    {
                        c'4
                        c'4
                        c'4
                    }
                }
                {
                    \time 2/4
                    {
                        c'4
                        c'4
                    }
                }
            }

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_exponent',
        )

    _class_name_abbreviation = ('RM', 'E')

    _human_readable_class_name = 'even-run rhythm-maker'

#    ### GALLERY EXAMPLES ###
#
#    _gallery_division_lists = (
#        [
#            (4, 16), (5, 16), (1, 2), (2, 12),
#            (1, 2), (2, 12), (4, 16), (5, 16),
#        ],
#        [
#            (1, 11), (3, 8), (3, 8), (3, 8),
#            (3, 8), (3, 8), (1, 11), (3, 8),
#        ],
#        [
#            (3, 15), (3, 15), (3, 16),
#            (5, 24), (5, 24), (5, 16),
#            (2, 12), (2, 12), (2, 8),
#            (3, 28), (3, 28), (3, 16),
#            (1, 9), (1, 9), (1, 8),
#        ],
#        [
#            (9, 16), (1, 5), (9, 16),
#            (1, 5), (9, 16), (9, 16),
#        ],
#        )
#
#    _gallery_example_wrappers = (
#        ExampleWrapper(
#            arguments={
#                },
#            division_lists=_gallery_division_lists,
#            ),
#        ExampleWrapper(
#            arguments={
#                'beam_specifier': BeamSpecifier(
#                    beam_divisions_together=True,
#                    ),
#                },
#            division_lists=_gallery_division_lists,
#            ),
#        ExampleWrapper(
#            arguments={
#                'beam_specifier': BeamSpecifier(
#                    beam_each_division=False,
#                    ),
#                },
#            division_lists=_gallery_division_lists,
#            ),
#        ExampleWrapper(
#            arguments={
#                'tie_specifier': TieSpecifier(
#                    tie_across_divisions=True,
#                    ),
#                },
#            division_lists=_gallery_division_lists,
#            ),
#        ExampleWrapper(
#            arguments={
#                'exponent': 1,
#                },
#            division_lists=_gallery_division_lists,
#            ),
#        )

    ### INITIALIZER ###

    def __init__(
        self,
        exponent=None,
        beam_specifier=None,
        duration_spelling_specifier=None,
        tie_specifier=None,
        ):
        if exponent is not None:
            assert mathtools.is_nonnegative_integer(exponent)
        RhythmMaker.__init__(
            self,
            beam_specifier=beam_specifier,
            duration_spelling_specifier=duration_spelling_specifier,
            tie_specifier=tie_specifier,
            )
        self._exponent = exponent

    ### SPECIAL METHODS ###

    def __call__(self, divisions, seeds=None):
        r'''Calls even-run rhythm-maker on `divisions`.

        ..  container:: example

            ::

                >>> maker = rhythmmakertools.EvenRunRhythmMaker()
                >>> divisions = [(4, 8), (3, 4), (2, 4)]
                >>> result = maker(divisions)
                >>> for selection in result:
                ...     selection
                Selection(Container("c'8 c'8 c'8 c'8"),)
                Selection(Container("c'4 c'4 c'4"),)
                Selection(Container("c'4 c'4"),)

        Returns a list of selections. Each selection holds a single container
        filled with notes.
        '''
        return RhythmMaker.__call__(
            self,
            divisions,
            seeds=seeds,
            )

    def __format__(self, format_specification=''):
        r'''Formats even run rhythm-maker.

        ..  container:: example

            ::

                >>> print(format(maker))
                rhythmmakertools.EvenRunRhythmMaker()

        Set `format_specification` to `''` or `'storage'`.

        Returns string.
        '''
        superclass = super(EvenRunRhythmMaker, self)
        return superclass.__format__(format_specification=format_specification)

    ### PRIVATE METHODS ###

    def _make_container(self, division):
        from abjad.tools import rhythmmakertools
        duration_spelling_specifier = self.duration_spelling_specifier
        if duration_spelling_specifier is None:
            duration_spelling_specifier = \
                rhythmmakertools.DurationSpellingSpecifier()
        forbidden_written_duration = \
            duration_spelling_specifier.forbidden_written_duration
        time_signature = indicatortools.TimeSignature(division)
        implied_prolation = time_signature.implied_prolation
        numerator, denominator = division
        denominator = mathtools.greatest_power_of_two_less_equal(denominator)
        assert mathtools.is_positive_integer_power_of_two(denominator)
        exponent = self.exponent or 0
        denominator_multiplier = 2 ** exponent
        denominator *= denominator_multiplier
        unit_duration = durationtools.Duration(1, denominator)
        if forbidden_written_duration is not None:
            multiplier = 1
            while forbidden_written_duration <= unit_duration:
                unit_duration /= 2
                multiplier *= 2
            numerator *= multiplier
        numerator *= denominator_multiplier
        notes = scoretools.make_notes(numerator * [0], [unit_duration])
        if implied_prolation == 1:
            result = scoretools.Container(notes)
        else:
            multiplier = implied_prolation
            result = scoretools.Tuplet(multiplier, notes)
        return result

    def _make_music(self, divisions, seeds):
        from abjad.tools import rhythmmakertools
        selections = []
        assert all(isinstance(x, tuple) for x in divisions)
        for duration_pair in divisions:
            container = self._make_container(duration_pair)
            selection = selectiontools.Selection(container)
            selections.append(selection)
        beam_specifier = self.beam_specifier
        if not beam_specifier:
            beam_specifier = rhythmmakertools.BeamSpecifier()
        if beam_specifier.beam_divisions_together:
            durations = []
            for selection in selections:
                duration = selection.get_duration()
                durations.append(duration)
            beam = spannertools.DuratedComplexBeam(
                durations=durations,
                span_beam_count=1,
                nibs_towards_nonbeamable_components=False,
                )
            components = []
            for selection in selections:
                components.extend(selection)
            attach(beam, components)
        elif beam_specifier.beam_each_division:
            for selection in selections:
                beam = spannertools.MultipartBeam()
                attach(beam, selection)
        return selections

    ### PUBLIC PROPERTIES ###

    @property
    def duration_spelling_specifier(self):
        r'''Gets duration spelling specifier of even-run rhythm-maker.

        ..  container:: example

            ::

                >>> specifier = rhythmmakertools.DurationSpellingSpecifier(
                ...     forbidden_written_duration=Duration(1, 4),
                ...     )
                >>> maker = rhythmmakertools.EvenRunRhythmMaker(
                ...     duration_spelling_specifier=specifier,
                ...     )

            ::

                >>> divisions = [(4, 8), (3, 4), (2, 4)]
                >>> music = maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     music,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> staff = maker._get_rhythmic_staff(lilypond_file)
                >>> f(staff)
                \new RhythmicStaff {
                    {
                        \time 4/8
                        {
                            c'8 [
                            c'8
                            c'8
                            c'8 ]
                        }
                    }
                    {
                        \time 3/4
                        {
                            c'8 [
                            c'8
                            c'8
                            c'8
                            c'8
                            c'8 ]
                        }
                    }
                    {
                        \time 2/4
                        {
                            c'8 [
                            c'8
                            c'8
                            c'8 ]
                        }
                    }
                }

        Returns duration spelling specifier or none.
        '''
        return self._duration_spelling_specifier

    @property
    def exponent(self):
        r'''Gets exponent of even-run rhythm-maker.

        ..  container:: example

            Makes even run of notes each equal in duration to
            ``1/(2**d)`` with ``d`` equal to the denominator of each division
            on which the rhythm-maker is called:

            ::

                >>> maker = rhythmmakertools.EvenRunRhythmMaker(
                ...     exponent=1,
                ...     )

            ::

                >>> divisions = [(4, 8), (3, 4), (2, 4)]
                >>> music = maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     music,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> staff = maker._get_rhythmic_staff(lilypond_file)
                >>> f(staff)
                \new RhythmicStaff {
                    {
                        \time 4/8
                        {
                            c'16 [
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16 ]
                        }
                    }
                    {
                        \time 3/4
                        {
                            c'8 [
                            c'8
                            c'8
                            c'8
                            c'8
                            c'8 ]
                        }
                    }
                    {
                        \time 2/4
                        {
                            c'8 [
                            c'8
                            c'8
                            c'8 ]
                        }
                    }
                }

        Defaults to none and interprets none equal to ``0``.

        Returns nonnegative integer or none.
        '''
        return self._exponent

    @property
    def tie_specifier(self):
        r'''Gets tie specifier of rhythm-maker.

        ..  container:: example

            Ties across divisions:

            ::

                >>> tie_specifier = rhythmmakertools.TieSpecifier(
                ...     tie_across_divisions=True,
                ...     )
                >>> maker = rhythmmakertools.EvenRunRhythmMaker(
                ...     tie_specifier=tie_specifier,
                ...     )

            ::

                >>> divisions = [(4, 8), (3, 4), (2, 4)]
                >>> music = maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     music,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> staff = maker._get_rhythmic_staff(lilypond_file)
                >>> f(staff)
                \new RhythmicStaff {
                    {
                        \time 4/8
                        {
                            c'8 [
                            c'8
                            c'8
                            c'8 ~ ]
                        }
                    }
                    {
                        \time 3/4
                        {
                            c'4
                            c'4
                            c'4 ~
                        }
                    }
                    {
                        \time 2/4
                        {
                            c'4
                            c'4
                        }
                    }
                }

        Returns boolean.
        '''
        return RhythmMaker.tie_specifier.fget(self)

    ### PUBLIC METHODS ###

    def reverse(self):
        r'''Reverses even-run rhythm-maker.

        ..  container:: example

            ::

                >>> maker = rhythmmakertools.EvenRunRhythmMaker()
                >>> reversed_maker = maker.reverse()

            ::

                >>> print(format(reversed_maker))
                rhythmmakertools.EvenRunRhythmMaker(
                    duration_spelling_specifier=rhythmmakertools.DurationSpellingSpecifier(
                        decrease_durations_monotonically=False,
                        ),
                    )

            ::

                >>> divisions = [(4, 8), (3, 4), (2, 4)]
                >>> music = maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     music,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> staff = maker._get_rhythmic_staff(lilypond_file)
                >>> f(staff)
                \new RhythmicStaff {
                    {
                        \time 4/8
                        {
                            c'8 [
                            c'8
                            c'8
                            c'8 ]
                        }
                    }
                    {
                        \time 3/4
                        {
                            c'4
                            c'4
                            c'4
                        }
                    }
                    {
                        \time 2/4
                        {
                            c'4
                            c'4
                        }
                    }
                }

        Returns new even-run rhythm-maker.
        '''
        from abjad.tools import rhythmmakertools
        duration_spelling_specifier = self.duration_spelling_specifier
        if duration_spelling_specifier is None:
            default = rhythmmakertools.DurationSpellingSpecifier()
            duration_spelling_specifier = default
        duration_spelling_specifier = duration_spelling_specifier.reverse()
        maker = new(
            self,
            duration_spelling_specifier=duration_spelling_specifier,
            )
        return maker



# -*- encoding: utf-8 -*-
import importlib
import pickle
from docutils import nodes
from docutils.parsers.rst import directives
from sphinx.util.compat import Directive
from abjad.docs.source._ext.abjad_book import visit_abjad_book_html
from abjad.docs.source._ext.abjad_book import visit_abjad_book_latex


class InheritanceException(Exception):
    pass


class abjad_lineage(nodes.General, nodes.Element):
    pass


class AbjadLineage(Directive):

    has_content = False
    required_arguments = 1
    optional_arguments = 0
    final_argument_whitespace = True
    option_spec = {}

    def run(self):
        from abjad.tools import documentationtools

        node = abjad_lineage()
        node.document = self.state.document
        env = self.state.document.settings.env
        parts = self.arguments[0].rpartition('.')
        module_name, class_name = str(parts[0]), str(parts[2])

        # Create a graph starting with the list of classes
        try:
            if module_name.startswith('abjad'):
                addresses = ('abjad',)
            elif module_name.startswith('experimental'):
                addresses = ('abjad', 'experimental')
            else:
                addresses = (module_name.partition('.')[0],)
                #raise InheritanceException(
                #    'Could not import class {!r} specified for '
                #    'inheritance diagram'.format(self.arguments[0]))
            # The following is a hack.
            # We need to guarantee that everything is imported before
            # the first diagram in the entire document is drawn,
            # otherwise the InheritanceGraph doesn't see anything... why?
            module = importlib.import_module(module_name)
            #lineage = documentationtools.InheritanceGraph(
            #    addresses=addresses,
            #    lineage_addresses=((module_name, class_name),)
            #    )
            lineage = documentationtools.InheritanceGraph(
                addresses=addresses,
                lineage_addresses=((module_name, class_name),)
                )
            graph = lineage.graphviz_graph

        except InheritanceException as err:
            return [node.document.reporter.warning(err.args[0],
                                                   line=self.lineno)]

        maximum_node_count = 30

        # begin pruning
        if maximum_node_count < len(graph.leaves):
            lineage = documentationtools.InheritanceGraph(
                addresses=addresses,
                lineage_addresses=((module_name, class_name),),
                lineage_prune_distance=2,
                )
            graph = lineage.graphviz_graph

        # keep pruning if still too many nodes
        if maximum_node_count < len(graph.leaves):
            lineage = documentationtools.InheritanceGraph(
                addresses=addresses,
                lineage_addresses=((module_name, class_name),),
                lineage_prune_distance=1,
                )
            graph = lineage.graphviz_graph

        # finally, revert to subclass-less version if still too many nodes
        if maximum_node_count < len(graph.leaves):
            lineage = documentationtools.InheritanceGraph(
                addresses=((module_name, class_name),),
                )

            def get_node_name(original_name):
                parts = original_name.split('.')
                name = [parts[0]]
                for part in parts[1:]:
                    if part != name[-1]:
                        name.append(part)
                if name[0] in ('abjad', 'experimental'):
                    return str('.'.join(name[2:]))
                return str('.'.join(name))

            node_name = get_node_name(module_name + '.' + class_name)
            graph = lineage.graphviz_graph
            graph_node = graph[node_name]
            graph_node.attributes['color'] = 'black'
            graph_node.attributes['fontcolor'] = 'white'
            graph_node.attributes['style'] = ('filled', 'rounded')

        node['code'] = pickle.dumps(graph)
        node['kind'] = 'graphviz'
        node['is_pickled'] = True
        node['keep_original'] = True

        return [node]


def skip(self, node):
    raise nodes.SkipNode


def setup(app):
    app.setup_extension('sphinx.ext.graphviz')
    app.add_node(
        abjad_lineage,
        latex=(visit_abjad_book_latex, None),
        html=(visit_abjad_book_html, None),
        text=(skip, None),
        man=(skip, None),
        texinfo=(skip, None))
    app.add_directive('abjad-lineage', AbjadLineage)


# -*- encoding: utf-8 -*-


def sort_named_pitch_carriers_in_expr(pitch_carriers):
    '''List named pitch carriers in `expr` sorted by
    numbered pitch-class:

    ::

        >>> notes = scoretools.make_notes([9, 11, 12, 14, 16], (1, 4))

    ::

        >>> pitchtools.sort_named_pitch_carriers_in_expr(notes)
        [Note("c''4"), Note("d''4"), Note("e''4"), Note("a'4"), Note("b'4")]

    The elements in `pitch_carriers` are not changed in any way.

    Returns list.
    '''
    from abjad.tools import pitchtools

    result = list(pitch_carriers[:])
    tmp = pitchtools.list_named_pitches_in_expr
    result.sort(
        key=lambda x:
        pitchtools.NumberedPitchClass(tmp(x)[0]).pitch_class_number
        )

    return result


# -*- encoding: utf-8 -*-
from abjad.tools import schemetools
from abjad.tools.spannertools.Spanner import Spanner


class StaffLinesSpanner(Spanner):
    r'''A staff lines spanner.

    ..  container:: example

        ::

            >>> staff = Staff("c'8 d'8 e'8 f'8")
            >>> spanner = spannertools.StaffLinesSpanner(lines=1)
            >>> attach(spanner, staff[1:3])
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> print(format(staff))
            \new Staff {
                c'8
                \stopStaff
                \once \override Staff.StaffSymbol.line-count = 1
                \startStaff
                d'8
                e'8
                \stopStaff
                \startStaff
                f'8
            }

    Stops and restarts staff on first leaf in spanner.

    Overrides ``line-count`` attribute of LilyPond ``Staff.StaffSymbol`` grob
    on first leaf in spanner.

    Stops and restarts staff on last leaf in spanner.

    Reverts ``line-count`` attribute of LilyPond ``Staff.StaffSymbol`` grob on
    last leaf in spanner.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_lines',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        lines=5,
        overrides=None,
        ):
        Spanner.__init__(
            self,
            overrides=overrides,
            )
        if isinstance(lines, int) and 0 < lines:
            self._lines = lines
        elif isinstance(lines, (tuple, list)) \
            and all(isinstance(x, (int, float)) for x in lines):
            self._lines = tuple(lines)
        else:
            message = 'must be integer or a sequence of integers: {!r}.'
            message = message.format(lines)
            raise ValueError(message)
        self._lines = lines

    ### PRIVATE METHODS ###

    def _copy_keyword_args(self, new):
        new._lines = self.lines

    def _format_after_leaf(self, leaf):
        result = []
        if self._is_my_last_leaf(leaf):
            result.append(r'\stopStaff')
            result.append(r'\startStaff')
        return result

    def _format_before_leaf(self, leaf):
        from abjad.tools import lilypondnametools
        result = []
        if self._is_my_first_leaf(leaf):
            result.append(r'\stopStaff')
            if isinstance(self.lines, int):
                override = lilypondnametools.LilyPondGrobOverride(
                    context_name='Staff',
                    grob_name='StaffSymbol',
                    is_once=True,
                    property_path='line-count',
                    value=self.lines,
                    )
                string = '\n'.join(override.override_format_pieces)
                result.append(string)
            else:
                override = lilypondnametools.LilyPondGrobOverride(
                    context_name='Staff',
                    grob_name='StaffSymbol',
                    is_once=True,
                    property_path='line-positions',
                    value=schemetools.SchemeVector(*self.lines),
                    )
                string = '\n'.join(override.override_format_pieces)
                result.append(string)
            result.append(r'\startStaff')
        return result

    ### PUBLIC PROPERTIES ###

    @property
    def lines(self):
        r'''Gets line of staff lines spanner.

        ::

            >>> staff = Staff("c'8 d'8 e'8 f'8")
            >>> spanner = spannertools.StaffLinesSpanner(lines=1)
            >>> attach(spanner, staff[1:3])
            >>> show(staff) # doctest: +SKIP

        ::

            >>> spanner.lines
            1

        Returns nonnegative integer.
        '''
        return self._lines



# -*- encoding: utf-8 -*-
from abjad.tools.mathtools.Ratio import Ratio


def test_mathtools_Ratio___repr___01():
    r'''Repr is evaluable.
    '''

    ratio_1 = Ratio(1, 2, -1)
    ratio_2 = eval(repr(ratio_1))

    assert isinstance(ratio_1, Ratio)
    assert isinstance(ratio_2, Ratio)
    assert ratio_1 is not ratio_2
    assert ratio_1 == ratio_2


# -*- encoding: utf-8 -*-
from abjad import *


def test_labeltools_color_leaves_in_expr_01():

    staff = Staff("cs'8. r8. s8. <c' cs' a'>8.")
    labeltools.color_leaves_in_expr(staff, 'red')

    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff {
            \once \override Accidental #'color = #red
            \once \override Beam #'color = #red
            \once \override Dots #'color = #red
            \once \override NoteHead #'color = #red
            \once \override Stem #'color = #red
            cs'8.
            \once \override Dots #'color = #red
            \once \override Rest #'color = #red
            r8.
            s8.
            \once \override Accidental #'color = #red
            \once \override Beam #'color = #red
            \once \override Dots #'color = #red
            \once \override NoteHead #'color = #red
            \once \override Stem #'color = #red
            <c' cs' a'>8.
        }
        '''
        )

    assert inspect_(staff).is_well_formed()


# -*- encoding: utf-8 -*-
from abjad import *


def test_pitchtools_list_numbered_inversion_equivalent_interval_classes_pairwise_01():

    staff = Staff("c'8 d'8 e'8 f'8 g'8 a'8 b'8 c''8")

    iecics = pitchtools.list_numbered_inversion_equivalent_interval_classes_pairwise(
        staff, wrap = False)

    assert iecics == [pitchtools.NumberedInversionEquivalentIntervalClass(2),
        pitchtools.NumberedInversionEquivalentIntervalClass(2), pitchtools.NumberedInversionEquivalentIntervalClass(1),         pitchtools.NumberedInversionEquivalentIntervalClass(2), pitchtools.NumberedInversionEquivalentIntervalClass(2),         pitchtools.NumberedInversionEquivalentIntervalClass(2), pitchtools.NumberedInversionEquivalentIntervalClass(1)]


def test_pitchtools_list_numbered_inversion_equivalent_interval_classes_pairwise_02():

    staff = Staff("c'8 d'8 e'8 f'8 g'8 a'8 b'8 c''8")

    iecics = pitchtools.list_numbered_inversion_equivalent_interval_classes_pairwise(
        staff, wrap = True)

    assert iecics == [pitchtools.NumberedInversionEquivalentIntervalClass(2),
        pitchtools.NumberedInversionEquivalentIntervalClass(2), pitchtools.NumberedInversionEquivalentIntervalClass(1),         pitchtools.NumberedInversionEquivalentIntervalClass(2), pitchtools.NumberedInversionEquivalentIntervalClass(2),         pitchtools.NumberedInversionEquivalentIntervalClass(2), pitchtools.NumberedInversionEquivalentIntervalClass(1),         pitchtools.NumberedInversionEquivalentIntervalClass(0)]


# -*- encoding: utf-8 -*-


def inventory_inversion_equivalent_named_interval_classes():
    '''Inventory inversion-equivalent diatonic interval-classes:

    ::

        >>> for dic in pitchtools.inventory_inversion_equivalent_named_interval_classes():
        ...     dic
        ...
        NamedInversionEquivalentIntervalClass('P1')
        NamedInversionEquivalentIntervalClass('aug1')
        NamedInversionEquivalentIntervalClass('+m2')
        NamedInversionEquivalentIntervalClass('+M2')
        NamedInversionEquivalentIntervalClass('+aug2')
        NamedInversionEquivalentIntervalClass('+dim3')
        NamedInversionEquivalentIntervalClass('+m3')
        NamedInversionEquivalentIntervalClass('+M3')
        NamedInversionEquivalentIntervalClass('+dim4')
        NamedInversionEquivalentIntervalClass('+P4')
        NamedInversionEquivalentIntervalClass('+aug4')

    There are 11 inversion-equivalent diatonic interval-classes.

    It is an open question as to whether octaves should be included.

    Returns list of inversion-equivalent diatonic interval-classes.
    '''
    from abjad.tools import pitchtools

    return [
        pitchtools.NamedInversionEquivalentIntervalClass('perfect', 1),
        pitchtools.NamedInversionEquivalentIntervalClass('augmented', 1),

        pitchtools.NamedInversionEquivalentIntervalClass('minor', 2),
        pitchtools.NamedInversionEquivalentIntervalClass('major', 2),
        pitchtools.NamedInversionEquivalentIntervalClass('augmented', 2),

        pitchtools.NamedInversionEquivalentIntervalClass('diminished', 3),
        pitchtools.NamedInversionEquivalentIntervalClass('minor', 3),
        pitchtools.NamedInversionEquivalentIntervalClass('major', 3),

        pitchtools.NamedInversionEquivalentIntervalClass('diminished', 4),
        pitchtools.NamedInversionEquivalentIntervalClass('perfect', 4),
        pitchtools.NamedInversionEquivalentIntervalClass('augmented', 4),
        ]


# -*- encoding: utf-8 -*-
from __future__ import print_function
import doctest
import importlib
import os
import sys
from abjad.tools import stringtools
from abjad.tools import systemtools
from abjad.tools.developerscripttools.DirectoryScript import DirectoryScript
try:
    from StringIO import StringIO
except ImportError:
    from io import StringIO


class RunDoctestsScript(DirectoryScript):
    r'''Runs doctests on all Python files in current directory recursively.

    ..  shell::

        ajv doctest --help

    '''

    ### PUBLIC PROPERTIES ###

    @property
    def alias(self):
        r'''Alias of script.

        Returns ``'doctest'``.
        '''
        return 'doctest'

    @property
    def long_description(self):
        r'''Long description of  script.

        Returns string or none.
        '''
        return None

    @property
    def scripting_group(self):
        r'''Scripting group of script.

        Returns none.
        '''
        return None

    @property
    def short_description(self):
        r'''Short description of script.

        Returns string.
        '''
        return 'Run doctests on all modules in current path.'

    @property
    def version(self):
        r'''Version of script.

        Returns float.
        '''
        return 1.0

    ### PUBLIC PROPERTIES ###

    def process_args(
        self,
        args=None,
        file_paths=None,
        print_to_terminal=True,
        ):
        r'''Processes `args`.

        Returns none when `print_to_terminal` is false.

        Returns string(s) when `print_to_terminal` is true.

        Returns none.
        '''
        assert not (args and file_paths)
        result = []
        globs = importlib.import_module('abjad').__dict__.copy()
        try:
            experimental_module = importlib.import_module('experimental')
            experimental_demos_module = importlib.import_module('experimental')
            globs.update(experimental_module.__dict__)
            globs.update(experimental_demos_module.__dict__)
        except:
            pass
        try:
            scoremanager_module = importlib.import_module('scoremanager')
            globs['scoremanager'] = scoremanager_module
        except:
            pass
        globs['print_function'] = print_function
        optionflags = (
            doctest.NORMALIZE_WHITESPACE |
            doctest.ELLIPSIS
            )
        if args and args.diff:
            optionflags = optionflags | doctest.REPORT_NDIFF
        total_failures = 0
        total_modules = 0
        total_tests = 0
        failed_file_paths = []
        error_messages = []
        if not file_paths:
            file_paths = []
            for dir_path, dir_names, file_names in os.walk(args.path):
                dir_names[:] = [x for x in dir_names
                    if not x.startswith('.')]
                for file_name in sorted(file_names):
                    if (file_name.endswith('.py') and
                        not file_name.startswith('test_') and
                        not file_name == '__init__.py'):
                        file_path = os.path.abspath(
                            os.path.join(dir_path, file_name))
                        file_paths.append(file_path)
        for file_path in file_paths:
            total_modules += 1
            relative_path = os.path.relpath(file_path)
            string_buffer = StringIO()
            with systemtools.RedirectedStreams(stdout=string_buffer):
                failure_count, test_count = doctest.testfile(
                    file_path,
                    module_relative=False,
                    globs=globs,
                    optionflags=optionflags,
                    )
            if failure_count:
                failed_file_paths.append(os.path.relpath(file_path))
                error_messages.append(string_buffer.getvalue())
                if print_to_terminal:
                    result_code = ''.join((
                        self.colors['RED'],
                        'FAILED',
                        self.colors['END'],
                        ))
                    print(relative_path, result_code)
                else:
                    result_code = 'FAILED'
                    string = '{} {}'.format(relative_path, result_code)
                    result.append(string)
            else:
                if print_to_terminal:
                    result_code = ''.join((
                        self.colors['BLUE'],
                        'OK',
                        self.colors['END'],
                        ))
                    print(relative_path, result_code)
                else:
                    result_code = 'OK'
                    string = '{} {}'.format(relative_path, result_code)
                    result.append(string)
            total_failures += failure_count
            total_tests += test_count
        if failed_file_paths:
            if print_to_terminal:
                print()
            else:
                result.append('')
            for error_message in error_messages:
                if print_to_terminal:
                    print(error_message)
                else:
                    result.append(error_message)
        for file_path in failed_file_paths:
            string = 'FAILED: {}'.format(file_path)
            if print_to_terminal:
                print(string)
            else:
                result.append(string)
        total_successes = total_tests - total_failures
        if print_to_terminal:
            print()
        else:
            result.append('')
        test_identifier = stringtools.pluralize('test', total_tests)
        module_identifier = stringtools.pluralize('module', total_modules)
        string = '{} of {} {} passed in {} {}.'
        string = string.format(
            total_successes,
            total_tests,
            test_identifier,
            total_modules,
            module_identifier,
            )
        if print_to_terminal:
            print(string)
            if total_successes == total_tests:
                sys.exit(0)
            else:
                sys.exit(1)
        else:
            result.append(string)
            return result


    def setup_argument_parser(self, parser):
        r'''Sets up argument `parser`.

        Returns none.
        '''
        parser.add_argument(
            'path',
            default=os.getcwd(),
            help='directory tree to be recursed over',
            nargs='?',
            )
        parser.add_argument(
            '--diff',
            action='store_true',
            help='print diff-like output on failed tests.',
            )



# -*- encoding: utf-8 -*-


def space_delimited_lowercase_to_upper_camel_case(string):
    '''Changes space-delimited lowercase `string` to uppercamelcase.

    ..  container:: example

        ::

            >>> string = 'bass figure alignment positioning'
            >>> stringtools.space_delimited_lowercase_to_upper_camel_case(string)
            'BassFigureAlignmentPositioning'

    Returns string.
    '''

    parts = string.split(' ')
    parts = [part.title() for part in parts]
    return ''.join(parts)


from abjad.tools.abctools import AbjadObject


class UpdateManager(AbjadObject):
    '''Update start offset, stop offsets and indicators everywhere in score.
    '''

    ### PRIVATE METHODS ###

    @staticmethod
    def _get_score_tree_state_flags(component):
        offsets_are_current = True
        indicators_are_current = True
        offsets_in_seconds_are_current = True
        parentage = component._get_parentage()
        for component in parentage:
            if offsets_are_current:
                if not component._offsets_are_current:
                    offsets_are_current = False
            if indicators_are_current:
                if not component._indicators_are_current:
                    indicators_are_current = False
            if offsets_in_seconds_are_current:
                if not component._offsets_in_seconds_are_current:
                    offsets_in_seconds_are_current = False
        return (
            offsets_are_current,
            indicators_are_current,
            offsets_in_seconds_are_current,
            )

    @staticmethod
    def _iterate_entire_score(component):
        from abjad.tools.topleveltools import iterate
        parentage = component._get_parentage(include_self=True)
        components = iterate(parentage.root).depth_first(
            capped=True,
            unique=True,
            forbid=None,
            direction='left',
            )
        return components

    @staticmethod
    def _update_all_indicators(component):
        r'''Updating indicators does not update offsets.
        On the other hand, getting an effective indicator does update
        offsets when at least one indicator of the appropriate type
        attaches to score.
        '''
        components = UpdateManager._iterate_entire_score(component)
        for component in components:
            for indicator in component._get_indicators(unwrap=False):
                if indicator.scope is not None:
                    assert hasattr(indicator, '_update_effective_context')
                    indicator._update_effective_context()
            component._indicators_are_current = True

    @staticmethod
    def _update_all_leaf_indices_and_measure_numbers(component):
        r'''Call only when updating offsets.
        No separate state flags exist for leaf indices or measure numbers.
        '''
        from abjad.tools import scoretools
        from abjad.tools.topleveltools import iterate
        parentage = component._get_parentage()
        score_root = parentage.root
        if isinstance(score_root, scoretools.Context):
            contexts = iterate(score_root).by_class(scoretools.Context)
            for context in contexts:
                for leaf_index, leaf in enumerate(
                    iterate(context).by_class(scoretools.Leaf)):
                    leaf._leaf_index = leaf_index
                for measure_index, measure in enumerate(
                    iterate(context).by_class(scoretools.Measure)):
                    measure_number = measure_index + 1
                    measure._measure_number = measure_number
        else:
            for leaf_index, leaf in enumerate(
                iterate(score_root).by_class(scoretools.Leaf)):
                leaf._leaf_index = leaf_index
            for measure_index, measure in enumerate(
                iterate(score_root).by_class(scoretools.Measure)):
                measure_number = measure_index + 1
                measure._measure_number = measure_number

    @staticmethod
    def _update_all_offsets(component):
        r'''Updating offsets does not update indicators.
        Updating offsets does not update offsets in seconds.
        '''
        components = UpdateManager._iterate_entire_score(component)
        for component in components:
            UpdateManager._update_component_offsets(component)
            component._offsets_are_current = True

    @staticmethod
    def _update_all_offsets_in_seconds(component):
        components = UpdateManager._iterate_entire_score(component)
        for component in components:
            UpdateManager._update_component_offsets_in_seconds(component)
            component._offsets_in_seconds_are_current = True

    @staticmethod
    def _update_component_offsets(component):
        from abjad.tools import durationtools
        from abjad.tools import scoretools
        previous = component._get_nth_component_in_time_order_from(-1)
        if previous is not None:
            start_offset = previous._stop_offset
        else:
            start_offset = durationtools.Offset(0)
        stop_offset = start_offset + component._get_duration()
        component._start_offset = start_offset
        component._stop_offset = stop_offset
        component._timespan._start_offset = start_offset
        component._timespan._stop_offset = stop_offset

    @staticmethod
    def _update_component_offsets_in_seconds(component):
        from abjad.tools import durationtools
        from abjad.tools import scoretools
        try:
            current_duration_in_seconds = \
                component._get_duration(in_seconds=True)
            previous = component._get_nth_component_in_time_order_from(-1)
            if previous is not None:
                component._start_offset_in_seconds = \
                    previous._get_timespan(in_seconds=True).stop_offset
            else:
                component._start_offset_in_seconds = durationtools.Offset(0)
            # this one case is possible for containers only
            if component._start_offset_in_seconds is None:
                raise MissingTempoError
            component._stop_offset_in_seconds = \
                component._start_offset_in_seconds + \
                current_duration_in_seconds
        except MissingTempoError:
            pass

    @staticmethod
    def _update_now(
        component,
        offsets=False,
        offsets_in_seconds=False,
        indicators=False,
        ):
        if component._is_forbidden_to_update:
            return
        parentage = component._get_parentage()
        for parent in parentage:
            if parent._is_forbidden_to_update:
                return
        state_flags = UpdateManager._get_score_tree_state_flags(component)
        offsets_are_current = state_flags[0]
        indicators_are_current = state_flags[1]
        offsets_in_seconds_are_current = state_flags[2]
        if offsets and not offsets_are_current:
            UpdateManager._update_all_offsets(component)
            UpdateManager._update_all_leaf_indices_and_measure_numbers(
                component)
        if offsets_in_seconds and not offsets_in_seconds_are_current:
            UpdateManager._update_all_offsets_in_seconds(component)
        if indicators and not indicators_are_current:
            UpdateManager._update_all_indicators(component)
            UpdateManager._update_all_offsets_in_seconds(component)


# -*- encoding: utf-8 -*-
from abjad import *


def test_sequencetools_repeat_elements_01():

    sequence_2 = list(sequencetools.repeat_elements(
        list(range(10)), [6, 7, 8], total=3))
    assert sequence_2 == [0, 1, 2, 3, 4, 5, [6, 6, 6], [7, 7, 7], [8, 8, 8], 9]


def test_sequencetools_repeat_elements_02():
    r'''Boundary cases.
    '''

    sequence_2 = list(sequencetools.repeat_elements(
        list(range(10)), [], total=99))
    assert sequence_2 == [0, 1, 2, 3, 4, 5, 6, 7, 8, 9]

    sequence_2 = list(sequencetools.repeat_elements(
        list(range(10)), list(range(10)), total=1))
    assert sequence_2 == [[0], [1], [2], [3], [4], [5], [6], [7], [8], [9]]

    sequence_2 = list(sequencetools.repeat_elements(
        list(range(10)), list(range(10)), total=2))
    assert sequence_2 == [[0, 0], [1, 1], [2, 2], [3, 3], [4, 4],
        [5, 5], [6, 6], [7, 7], [8, 8], [9, 9]]

    sequence_2 = list(sequencetools.repeat_elements(
        list(range(10)), [6, 7, 8], total=0))
    assert sequence_2 == [0, 1, 2, 3, 4, 5, [], [], [], 9]



# -*- encoding: utf-8 -*-
from abjad.tools import mathtools
from abjad.tools import sequencetools
from abjad.tools.abctools import AbjadValueObject
from abjad.tools.topleveltools import new


class InciseSpecifier(AbjadValueObject):
    r'''Incision specifier.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_body_ratio',
        '_fill_with_notes',
        '_incise_divisions',
        '_incise_output',
        '_prefix_talea',
        '_prefix_lengths',
        '_suffix_talea',
        '_suffix_lengths',
        '_talea_denominator',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        incise_divisions=False,
        incise_output=False,
        prefix_talea=(-1,),
        prefix_lengths=(0, 1),
        suffix_talea=(-11,),
        suffix_lengths=(1,),
        talea_denominator=32,
        body_ratio=None,
        fill_with_notes=True,
        ):
        assert isinstance(incise_divisions, bool)
        self._incise_divisions = incise_divisions
        assert isinstance(incise_output, bool)
        self._incise_output = incise_output
        assert self._is_integer_tuple(prefix_talea)
        self._prefix_talea = prefix_talea
        assert self._is_length_tuple(prefix_lengths)
        self._prefix_lengths = prefix_lengths
        assert self._is_integer_tuple(suffix_talea)
        self._suffix_talea = suffix_talea
        assert self._is_length_tuple(suffix_lengths)
        self._suffix_lengths = suffix_lengths
        assert mathtools.is_nonnegative_integer_power_of_two(talea_denominator)
        self._talea_denominator = talea_denominator
        if body_ratio is not None:
            body_ratio = mathtools.Ratio(body_ratio)
        self._body_ratio = body_ratio
        assert isinstance(fill_with_notes, bool)
        self._fill_with_notes = fill_with_notes

    ### PRIVATE METHODS ###

    @staticmethod
    def _is_integer_tuple(expr):
        if expr is None:
            return True
        if all(isinstance(x, int) for x in expr):
            return True
        return False

    @staticmethod
    def _is_length_tuple(expr):
        if expr is None:
            return True
        if mathtools.all_are_nonnegative_integer_equivalent_numbers(expr):
            if isinstance(expr, tuple):
                return True
        return False

    @staticmethod
    def _reverse_tuple(expr):
        if expr is not None:
            return tuple(reversed(expr))

    @staticmethod
    def _rotate_tuple(expr, n):
        if expr is not None:
            return tuple(sequencetools.rotate_sequence(expr, n))

    @staticmethod
    def _to_tuple(expr):
        if isinstance(expr, list):
            expr = tuple(expr)
        return expr

    ### PUBLIC PROPERTIES ###

    @property
    def body_ratio(self):
        r'''Gets body ratio of incise specifier.

        ..  container:: example

            Sets `body_ratio` to divide middle part proportionally:

            ::

                >>> incise_specifier = rhythmmakertools.InciseSpecifier(
                ...     incise_divisions=True,
                ...     prefix_talea=(-1,),
                ...     prefix_lengths=(0, 1),
                ...     suffix_talea=(-1,),
                ...     suffix_lengths=(1,),
                ...     talea_denominator=16,
                ...     body_ratio=(1, 1),
                ...     )
                >>> maker = rhythmmakertools.IncisedRhythmMaker(
                ...     incise_specifier=incise_specifier,
                ...     )

            ::

                >>> divisions = 4 * [(5, 16)]
                >>> music = maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     music,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> staff = maker._get_rhythmic_staff(lilypond_file)
                >>> f(staff)
                \new RhythmicStaff {
                    {
                        \time 5/16
                        c'8 [
                        c'8 ]
                        r16
                    }
                    {
                        r16
                        c'16. [
                        c'16. ]
                        r16
                    }
                    {
                        c'8 [
                        c'8 ]
                        r16
                    }
                    {
                        r16
                        c'16. [
                        c'16. ]
                        r16
                    }
                }

        Defaults to none.

        Returns ratio or none.
        '''
        return self._body_ratio

    @property
    def fill_with_notes(self):
        r'''Is true when rhythm-maker should fill divisions with notes.
        Otherwise false.

        Defaults to true.

        Returns boolean.
        '''
        return self._fill_with_notes

    @property
    def incise_divisions(self):
        r'''Is true when rhythm-maker should incise every division.
        Otherwise false.

        Defaults to false.

        Returns boolean.
        '''
        return self._incise_divisions

    @property
    def incise_output(self):
        r'''Is true when rhythm-maker should incise first and last divisions.
        Otherwise false.

        Defaults to false.

        Returns boolean.
        '''
        return self._incise_output

    @property
    def prefix_lengths(self):
        r'''Gets prefix lengths of incision specifier.

        Returns tuple or none.
        '''
        return self._prefix_lengths

    @property
    def prefix_talea(self):
        r'''Gets prefix talea of incision specifier.

        Returns tuple or none.
        '''
        return self._prefix_talea

    @property
    def suffix_lengths(self):
        r'''Gets suffix lengths of incision specifier.

        Returns tuple or none.
        '''
        return self._suffix_lengths

    @property
    def suffix_talea(self):
        r'''Gets suffix talea of incision specifier.

        Returns tuple or none.
        '''
        return self._suffix_talea

    @property
    def talea_denominator(self):
        r'''Gets talea denominator of incision specifier.

        Returns positive integer-equivalent number.
        '''
        return self._talea_denominator

    ### PUBLIC METHODS ###

    def reverse(self):
        r'''Reverses incision specifier.

        Returns new incision specifier.
        '''
        from abjad.tools import rhythmmakertools
        prefix_lengths = rhythmmakertools.RhythmMaker._reverse_tuple(
            self.prefix_lengths)
        prefix_talea = rhythmmakertools.RhythmMaker._reverse_tuple(
            self.prefix_talea)
        suffix_lengths = rhythmmakertools.RhythmMaker._reverse_tuple(
            self.suffix_lengths)
        suffix_talea = rhythmmakertools.RhythmMaker._reverse_tuple(
            self.suffix_talea)
        maker = new(
            self,
            prefix_lengths=prefix_lengths,
            prefix_talea=prefix_talea,
            suffix_lengths=suffix_lengths,
            suffix_talea=suffix_talea,
            )
        return maker

    def rotate(self, n=0):
        r'''Rotates incision specifier.

        ..  container:: example

            ::

                >>> incise_specifier = rhythmmakertools.InciseSpecifier(
                ...     incise_divisions=True,
                ...     prefix_talea=(-1,),
                ...     prefix_lengths=(0, 2, 1),
                ...     suffix_talea=(-1, 1),
                ...     suffix_lengths=(1, 0, 0),
                ...     talea_denominator=16,
                ...     body_ratio=(1, 1),
                ...     )

            ::

                >>> print(format(incise_specifier.rotate(1)))
                rhythmmakertools.InciseSpecifier(
                    incise_divisions=True,
                    incise_output=False,
                    prefix_talea=(-1,),
                    prefix_lengths=(1, 0, 2),
                    suffix_talea=(1, -1),
                    suffix_lengths=(0, 1, 0),
                    talea_denominator=16,
                    body_ratio=mathtools.Ratio(1, 1),
                    fill_with_notes=True,
                    )

        Returns new incision specifier.
        '''
        from abjad.tools import rhythmmakertools
        prefix_lengths = rhythmmakertools.RhythmMaker._rotate_tuple(
            self.prefix_lengths, n)
        prefix_talea = rhythmmakertools.RhythmMaker._rotate_tuple(
            self.prefix_talea, n)
        suffix_lengths = rhythmmakertools.RhythmMaker._rotate_tuple(
            self.suffix_lengths, n)
        suffix_talea = rhythmmakertools.RhythmMaker._rotate_tuple(
            self.suffix_talea, n)
        maker = new(
            self,
            prefix_lengths=prefix_lengths,
            prefix_talea=prefix_talea,
            suffix_lengths=suffix_lengths,
            suffix_talea=suffix_talea,
            )
        return maker



# -*- encoding: utf-8 -*-
import abc
import inspect
from abjad.tools import durationtools
from abjad.tools.abctools import AbjadObject
from abjad.tools.topleveltools import new


class QEvent(AbjadObject):
    r'''Abstract base class from which concrete ``QEvent`` subclasses
    inherit.

    Represents an attack point to be quantized.

    All ``QEvents`` possess a rational offset in milliseconds,
    and an optional index for disambiguating events which fall
    on the same offset in a ``QGrid``.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_index',
        '_offset',
        )

    ### INITIALIZER ###

    @abc.abstractmethod
    def __init__(self, offset=0, index=None):
        offset = durationtools.Offset(offset)
        self._offset = offset
        self._index = index

    ### SPECIAL METHODS ###

    def __getstate__(self):
        r'''Gets state.
        '''
        state = {}
        for current_class in inspect.getmro(type(self)):
            if hasattr(current_class, '__slots__'):
                for slot in current_class.__slots__:
                    if slot not in state:
                        state[slot] = getattr(self, slot)
        return state

    def __lt__(self, expr):
        r'''Is true when `epxr` is a q-event with offset greater than that of this
        q-event. Otherwise false.

        Returns boolean.
        '''
        if type(self) == type(self):
            if self.offset < expr.offset:
                return True
        return False

    def __setstate__(self, state):
        r'''Sets state.
        '''
        for key, value in state.items():
            setattr(self, key, value)

    ### PRIVATE PROPERTIES ###

    @property
    def _repr_specification(self):
        return new(
            self._storage_format_specification,
            is_indented=False,
            )

    @property
    def _storage_format_specification(self):
        from abjad.tools import systemtools
        return systemtools.StorageFormatSpecification(
            self,
            keywords_ignored_when_false=(
                'attachments',
                ),
            )

    ### PUBLIC PROPERTIES ###

    @property
    def index(self):
        r'''The optional index, for sorting QEvents with identical offsets.
        '''
        return self._index

    @property
    def offset(self):
        r'''The offset in milliseconds of the event.
        '''
        return self._offset


# -*- encoding: utf-8 -*-
from abjad.tools import scoretools
from abjad.tools.spannertools.Beam import Beam


class MultipartBeam(Beam):
    r'''A multipart beam.

    ..  container:: example

        ::

            >>> staff = Staff("c'8 d'8 e'4 f'8 g'8 r4")
            >>> set_(staff).auto_beaming = False
            >>> show(staff) # doctest: +SKIP

        ::

            >>> beam = spannertools.MultipartBeam()
            >>> attach(beam, staff[:])
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> print(format(staff))
            \new Staff \with {
                autoBeaming = ##f
            } {
                c'8 [
                d'8 ]
                e'4
                f'8 [
                g'8 ]
                r4
            }

    Avoids rests.

    Avoids large-duration notes.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        )

    ### PRIVATE METHODS ###

    def _get_lilypond_format_bundle(self, leaf):
        lilypond_format_bundle = self._get_basic_lilypond_format_bundle(leaf)
        if not self._is_beamable_component(leaf):
            return lilypond_format_bundle
        direction_string = ''
        if self.direction is not None:
            direction_string = '{} '.format(self.direction)
        beamable_leaf_count = 0
        for _ in self._leaves:
            if self._is_beamable_component(_):
                beamable_leaf_count += 1
        if 2 <= beamable_leaf_count:
            previous_leaf = leaf._get_leaf(-1)
            if previous_leaf not in self._leaves:
                previous_leaf = None
            next_leaf = leaf._get_leaf(1)
            if next_leaf not in self._leaves:
                next_leaf = None
            start_piece = None
            stop_piece = None
            if self._is_my_first_leaf(leaf):
                if next_leaf is not None:
                    if self._is_beamable_component(next_leaf):
                        start_piece = '{}['.format(direction_string)
            else:
                if previous_leaf is not None:
                    if not self._is_beamable_component(previous_leaf):
                        if self._is_beamable_component(next_leaf):
                            start_piece = '{}['.format(direction_string)
            if self._is_my_last_leaf(leaf):
                if previous_leaf is not None:
                    if self._is_beamable_component(previous_leaf):
                        stop_piece = ']'
            else:
                if self._is_beamable_component(previous_leaf):
                    next_leaf = leaf._get_leaf(1)
                    if next_leaf is not None:
                        if not self._is_beamable_component(next_leaf):
                            stop_piece = ']'
            if start_piece and stop_piece:
                lilypond_format_bundle.right.spanner_starts.extend([
                    start_piece, stop_piece])
            elif start_piece:
                lilypond_format_bundle.right.spanner_starts.append(start_piece)
            elif stop_piece:
                lilypond_format_bundle.right.spanner_stops.append(stop_piece)
        return lilypond_format_bundle
